<?PHP
class Bio_BioMartQuery
{
	private static $martServiceURL = 'http://www.biomart.org/biomart/martservice?query=';
	private $keyword;
	private $id;

	private $query;
	private $queryXML;

	public function __construct($keyword)
	{
		$this->keyword = $keyword;
		$this->getKeywordID();
		$this->query    = array();
		$this->queryXML = '';
	}

	protected function getKeywordID()
	{
		$row 	= array();
		$query	= "SELECT id FROM entrez_search WHERE keyword='" . $this->keyword . "'";
		$result = DB::executeQuery($query);
		$row    = $result->next();
		if(!is_numeric($row['id'])) {
			throw new Exception('Unable to get ID for query, invalid keyword: [' . $this->keyword . ']');
		}
		$this->id = $row['id'];
	}

	public function setDataset($name)
	{
		$this->query['dataset'] = $name;
	}

	public function addFilter($name, $value)
	{
		$this->query['filters'][$name] = $value;
 	}

	public function addAttribute($name)
	{
		$this->query['attributes'][] = $name;
	}

	protected function buildQuery()
	{
		if(empty($this->query['dataset']) || empty($this->query['filters']) || empty($this->query['attributes'])) {
			throw new Exception("Invalid query! Please specify the dataset, and set at least one filter and attribute.");
		}
		$this->queryXML	= "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
						. "<!DOCTYPE Query>"
						. "<Query  virtualSchemaName=\"default\" formatter=\"TSV\" header=\"0\" uniqueRows=\"1\" count=\"\" datasetConfigVersion=\"0.6\">"
						. "<Dataset name=\"" . $this->query['dataset'] . "\" interface=\"default\">";
		foreach($this->query['filters'] as $name => $value) {
			$this->queryXML.= "<Filter name=\"" . $name . "\" value=\"" . $value . "\"/>";
		}
		foreach($this->query['attributes'] as $name) {
			$this->queryXML.= "<Attribute name=\"" . $name . "\"/>";
		}
		$this->queryXML.= "</Dataset>"
						. "</Query>";
	}

	public function execute()
	{
		$this->buildQuery();
		$socket = Socket::getInstance();
		$queryString = self::$martServiceURL . urlencode($this->queryXML);
		$socket->doHTTPGet($queryString);
		$response = trim($socket->getHTTPBody());
		$socket->disconnect();
		return $response;
	}

}