<?PHP
class Bio_EntrezFetch
{
	private static $eFetchURL 	= 'http://eutils.ncbi.nlm.nih.gov/entrez/eutils/efetch.fcgi?';
	private $database 			= 'nuccore';
	private $retriveMode		= 'xml';
	private $retriveType		= 'gb'; // GenBank
	private $URL;

	private $keyword;
	private $id;
	private $socket;

	public function __construct($keyword)
	{
		$this->keyword = $keyword;
		$this->socket  = Socket::getInstance();
		$this->setURL();
	}

	protected function getKeywordID()
	{
		$row 	= array();
		$query	= "SELECT id FROM entrez_search WHERE keyword='" . $this->keyword . "'";
		$result = DB::executeQuery($query);
		$row    = $result->next();
		if(!is_numeric($row['id'])) {
			throw new Exception('Unable to begin fetching, invalid keyword: [' . $this->keyword . ']');
		}
		$this->id = $row['id'];
	}

	protected function setURL()
	{
		$this->URL = self::$eFetchURL . 'db=' . $this->database . '&retmode=' . $this->retriveMode . '&rettype=' . $this->retriveType;
	}

	protected function fetchRow($id)
	{
		// assign the url
		$URL = $this->URL . '&id=' . $id;

		// open the socket
		$this->socket->doHTTPGet($URL);

		// parse xml
		$xml     = '';
		$xmlText = $this->socket->getHTTPBody();
		try {
			$xml = new SimpleXMLElement($xmlText);
		} catch (Exception $e) {
			Log::screenLog('Invalid XML.');
		}

		// get species name
		$organism   = trim((string)$xml->GBSeq->GBSeq_organism);
		$chromosome = '';
		$gene		= '';
		$ext_db		= '';
		$ext_ref	= '';
		if(strlen($organism) > 0) {
			foreach ($xml->xpath('///GBFeature') as $feature) {
				if(strtolower($feature->GBFeature_key) == 'gene') {
					foreach($feature->GBFeature_quals->GBQualifier as $qualifier) {
						if(strtolower($qualifier->GBQualifier_name) == 'gene') {
							$gene = (string)$qualifier->GBQualifier_value;
						} elseif (strtolower($qualifier->GBQualifier_name) == 'db_xref') {
							$value = explode(':', (string)$qualifier->GBQualifier_value);
							if( ( ($organism == 'Homo sapiens') && ($value[0] == 'HGNC') )
								||
							    ( ($organism == 'Mus musculus') && ($value[0] == 'MGI')  ) ) {
								$ext_db  = $value[0];
								$ext_ref = $value[1];
							}
						}
					}
				} elseif(strtolower($feature->GBFeature_key) == 'source') {
					foreach($feature->GBFeature_quals->GBQualifier as $qualifier) {
						if(strtolower($qualifier->GBQualifier_name) == 'chromosome') {
							$chromosome = (string)$qualifier->GBQualifier_value;
							break;
						}
					}
				}
				if((strlen($chromosome) > 0) && (strlen($gene) > 0)) {
					break;
				}
			}
		}
		if((($organism=='Homo sapiens') || ($organism=='Mus musculus')) && (strlen($chromosome) > 0) && (strlen($gene) > 0) && (strlen($gene) <= 20)) {
			$organismId = ($organism == 'Homo sapiens') ? 1 : 2;
			$query 	= "SELECT id FROM entrez_fetch WHERE "
					. "  search_id='" . $this->id . "' AND "
					. "  organism_id='" . $organismId . "' AND "
					. "  LOWER(gene)='" . strtolower($gene) . "'";
			$result = DB::executeQuery($query);
			if($result->getRecordCount() == 0) {
				Log::screenLog($organism . " | " . $chromosome . " | " . $gene . " | " . $ext_db . " | " . $ext_ref);
				// save the log
				$query 	= "INSERT INTO entrez_fetch (search_id, organism_id, chromosome, gene, db_name, db_xref) VALUES "
						. "('" . $this->id . "', '" . $organismId . "', '" . $chromosome . "', '" . $gene . "', "
						. "'" . $ext_db . "', '" . $ext_ref . "')";
				DB::executeInsert($query);
			}
		}

		// disconnect
		$this->socket->disconnect();

		// set status to fetched
		$query  = "UPDATE entrez_search_id SET fetched='1' WHERE search_id='" . $this->id . "' AND ncbi_id='" . $id . "'";
		DB::executeUpdate($query);
	}

	public function fetchAll()
	{
		$this->getKeywordID();
		$query	= "SELECT ncbi_id FROM entrez_search_id WHERE search_id='" . $this->id . "' AND fetched='0'";
		$result = DB::executeQuery($query);
		$j = 1;
		Log::addLog("Parsing query XML entries from NCBI server started with keyword: [" . $this->keyword . "]");
		while(($row = $result->next()) != false) {
			$this->fetchRow($row['ncbi_id']);
			if(($j % 100) == 0) {
				Log::screenLog(date('Y.m.d. H:i:s') . ": " . $j . " records parsed in sum.");
			}
			$j++;
		}
		Log::addLog("Parsing query XML entries from NCBI server ended with keyword: [" . $this->keyword . "] " . $j . " rows parsed");
		Log::screenLog("<strong>Finished: " . date('Y.m.d. H:i:s') . ": " . $j . " records parsed.</strong>");
	}

}