<?PHP
class Bio_EntrezSearch
{
	private static $eSearchURL = 'http://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi?';
	private $database = 'nuccore';

	private $keyword;
	private $recordCount;

	public function __construct($keyword)
	{
		$this->keyword = $keyword;
	}

	private function getURL()
	{
		return self::$eSearchURL . 'db=' . $this->database . '&term=' . $this->keyword;
	}

	public function getRecordCount()
	{
		// download the stream
		$socket = Socket::getInstance();
		$socket->doHTTPGet($this->getURL());

		// parse the XML
		$xml = new SimpleXMLElement($socket->getHTTPBody());

		$this->recordCount = (int)$xml->Count;
	}

	public function getIDs()
	{
		// prepare the table
		$query	= "TRUNCATE TABLE entrez_search_id";
		DB::executeUpdate($query);

		// do the things
		$socket = Socket::getInstance();
		$paging = 1000;
		$i      = 0;
		$sum	= 0;
		while($i < $this->recordCount) {
			// grab the proper URL
			$URL = $this->getURL() . '&retstart=' . $i . '&retmax=' . $paging;
			$socket->doHTTPGet($URL);

			// parse xml
			$xml    = new SimpleXMLElement($socket->getHTTPBody());
			$subSum = 0;
			foreach($xml->IdList->Id as $id) {
				$query = "INSERT INTO entrez_search_id (search_id, ncbi_id) VALUES ('1', '" . $id . "')";
				DB::executeInsert($query);
				$subSum++;
			}
			print date('Y.m.d. H:i:s') . ": " . $subSum . " records inserted<br />\n";
			flush();
			ob_flush();
			$sum += $subSum;
			$i += $paging;
		}
		print "<strong>" . date('Y.m.d. H:i:s') . ": " . $sum . " records inserted totally.</strong>\n";
	}

	public function doSearch()
	{
		$this->getRecordCount();
		if($this->recordCount > 0) {
			// save the search information
			$query 	= "UPDATE entrez_search SET "
					. "items='" . $this->recordCount . "', date_start='" . date('Y-m-d H:i:s') . "', "
					. "date_end='0000-00-00 00:00:00' "
					. "  WHERE keyword='" . $this->keyword . "'";
			DB::executeUpdate($query);

			Log::addLog("Fetching gene ID's from NCBI server started with keyword: [" . $this->keyword . "]");

			$this->getIDs();

			$query  = "UPDATE entrez_search SET date_end='" . date('Y-m-d H:i:s') . "' WHERE keyword='" . $this->keyword . "'";
			DB::executeUpdate($query);

			Log::addLog("Fetching gene ID's from NCBI server ended with keyword: [" . $this->keyword . "]");
		} else {
			print "This query returned 0 records.";
		}
	}

}