<?PHP
function __autoload($className) {
	if(strstr($className, 'Exception')) {
		require_once(__UTAS__ . DIRECTORY_SEPARATOR . 'Exception' . DIRECTORY_SEPARATOR . str_replace('Exception', '', $className) . '.php');
	} else {
		require_once(__UTAS__ . DIRECTORY_SEPARATOR . str_replace('_', DIRECTORY_SEPARATOR, $className) . '.php');
	}
}

class Utas {

	// paths
	protected $directory;
	protected $siteRoot;
	protected $siteClass;

	// URLS
	protected $siteURL;

	public function __construct($config = array(), $debug = false) {

		$this->directory = dirname(__FILE__);
		define('__UTAS__', $this->directory);
		$this->siteRoot  = str_replace(DIRECTORY_SEPARATOR . 'lib' . DIRECTORY_SEPARATOR . 'Utas', '', $this->directory);
		$this->siteClass = $this->siteRoot . DIRECTORY_SEPARATOR . 'lib' . DIRECTORY_SEPARATOR . 'class';
 		$this->siteURL   = str_replace('/admin', '', str_replace('/' . basename($_SERVER['PHP_SELF']), '', 'http://' . $_SERVER['HTTP_HOST'] . $_SERVER['PHP_SELF']));

		clearstatcache();

		if($config['session'] == true) {
			// start session
			session_start();
		}

		if(strlen($config['database']) > 0) {
			// connect to database
			preg_match('/^(mysqli):\/\/(\w+)(:\w+)?@(\w+)(:\w+)?\/(\w+)$/', $config['database'], $results);
			require_once($this->directory . DIRECTORY_SEPARATOR . 'Database/DB.php');
			try {
				DB::connect($results[4], $results[2], substr($results[3], 1), $results[6], substr($results[5], 1));
			} catch (Exception $e) {
				$this->exception($e);
			}
		}

		// logging service
		require_once($this->directory . DIRECTORY_SEPARATOR . 'Log.php');
		Log::initialize();

		// set debugging
		if($debug) {
			define('__DEBUG__', true);
		}

		// load additional functions
		require_once($this->siteRoot . DIRECTORY_SEPARATOR . 'lib' . DIRECTORY_SEPARATOR . 'functions.php');
	}

	public function getUtasDirectory() {
		return $this->directory;
	}

	public function getSiteRoot() {
		return $this->siteRoot;
	}

	public function getSiteClassDirectory() {
		return $this->siteClass;
	}

	public function getSiteURL() {
		return $this->siteURL;
	}

	public function exception(Exception $e) {
		print "<strong>Exception:</strong>: " . $e->getMessage() . "<br />\n"
			. "<pre>\n" . $e->getTraceAsString() . "</pre>\n";
	}

}
